package script

/**
 *
 * Скрипт для создания или обновления релиза в gitea для всех релизных тегов текущего коммита.
 * Ищет описание релиза в специальной папке, после чего добавляет его к релизу, если находит.
 *
 * Собирает докер образ, после чего из него копирует указанные как артефакты файлы и прикрепляет к релизу.
 *
 * Для работы требуется токен API Gitea и список нужных файлов, либо флаг того, что список может быть пустым.
 */


@GrabResolver(name='gitea', root='https://git.tswf.io/api/packages/public-repos/maven')
@Grapes([
        @Grab(group='io.tswf.groovy.better-groovy', module='better-groovy-scripting-shell', version='2.0.2-SNAPSHOT', changing = true),
        @Grab(group='io.tswf.groovy.better-groovy', module='better-groovy-scripting-gitea', version='2.0.2-SNAPSHOT', changing = true)
])
import io.tswf.gitea.parser.repository.GiteaRepositoryMetadataParser
import io.tswf.groovy.bettergroovy.scripting.v2.git.Git
import io.tswf.groovy.bettergroovy.scripting.v2.gitea.Gitea
import util.CIProperties
import util.ReleaseArtifactNames
import util.Dockerfiles
import util.DockerBuildCommandFactory
import util.GitTags
import util.GlobalProperties
import util.ReleaseArtifacts
import util.ReleaseDescriptionReader
import util.ScriptLog

println """
#############################################################################
                       Publish Release Artifacts Script
#############################################################################
"""

GlobalProperties.loadGlobalProperties()

def dockerFiles = Dockerfiles.getReleaseDockerfiles()

def isAnyArtifactRequiredForRelease = CIProperties.findProperty("gitea.release.artifacts.is-required")
    .map { it.toBoolean() }
    .orElse(true)

def releaseTags = GitTags.getReleaseTags()

if (releaseTags.isNullOrEmpty()) {
    ScriptLog.printf "There is no release tags to publish!"
    System.exit(1)
}

def giteaHost = CIProperties.findProperty("gitea.origin") // В рамках обратной совместимости
    .orElseGet {
        CIProperties.getProperty("gitea.host") // Актуальное название
    }

def giteaToken = CIProperties.getProperty("gitea.token")

def giteaRepositoryParser = new GiteaRepositoryMetadataParser()
def giteaRepositoryMetadata = giteaRepositoryParser.parseRepositoryMetadataFromOrigin(giteaHost)

def artifactLocalFiles = new ArrayList<File>()

for (def dockerfileName : dockerFiles) {
    ScriptLog.printf "Collecting artifacts from dockerfile named '${dockerfileName}'"

    // Собираем докер образ и поднимаем контейнер, чтобы далее извлечь из него нужные артефакты.
    def temporaryDockerImage = Random.randomString(40)
    def temporaryDockerContainerName = "build_in_docker_${Random.randomString(25)}"

    // Подчищаем за собою
    addShutdownHook {
        println "Removing temporary docker container '${temporaryDockerContainerName}' and image '${temporaryDockerImage}'"
        sh "docker rm -f ${temporaryDockerContainerName}"
        sh "docker image rm -f ${temporaryDockerImage}"
    }

    def temporaryBuildDir = new File("./gitea-release-publication/tmp/build/${Random.randomString(10)}/")
    temporaryBuildDir.mkdirsUnsafe()
    temporaryBuildDir.deleteOnExit()

    def dockerBuildCommand = DockerBuildCommandFactory.getBuildCommand(dockerfileName, "-t ${temporaryDockerImage}")
    sh dockerBuildCommand
    sh "docker create -it --name ${temporaryDockerContainerName} ${temporaryDockerImage} sh"

    def artifactsToPublish = ReleaseArtifacts.getArtifacts(dockerfileName)

    if (artifactsToPublish.isEmpty()) {
        ScriptLog.printf "There is no artifacts was configured for dockerfile named '${dockerfileName}'"
    } else {
        ScriptLog.printf "Copying artifacts for dockerfile named '${dockerfileName}': ${artifactsToPublish.join('\n')}"
    }

    // Копируем все артефакты для публикации из временного докер контейнера
    artifactsToPublish.stream()
        .map { artifactToPublishAbsolutePath ->
            def artifactSimpleName = artifactToPublishAbsolutePath.split("/").last()

            def artifactLocalFile = temporaryBuildDir.subFile(artifactSimpleName)
            sh "docker cp ${temporaryDockerContainerName}:${artifactToPublishAbsolutePath} ${artifactLocalFile.path}"

            artifactLocalFile
        }

        // Переименовываем файл, если это настроено
        .map { artifactFile ->
            ReleaseArtifactNames.getNewArtifactName(artifactFile, dockerfileName)
                .map { newName ->
                    def newFile = artifactFile.parentFile.subFile(newName)
                    ScriptLog.printf "Renaming artifact '${artifactFile.absoluteCanonicalFile.path}' to '${newName}' for dockerfile named '${dockerfileName}'"
                    artifactFile.renameTo(newFile)

                    newFile
                }
                .orElse(artifactFile)
        }

        .forEach {
            artifactLocalFiles << it
        }
}

ScriptLog.printf "Total ${artifactLocalFiles.size()} artifacts collected: ${artifactLocalFiles}"

if (artifactLocalFiles.isEmpty() && isAnyArtifactRequiredForRelease) {
    ScriptLog.printf "Release artifacts required, but no one found!"
    System.exit(1)
}

def giteaRepository = Gitea.getCIRepositoryUtilityV1(giteaToken, giteaRepositoryMetadata)

// Создаем релизы в гитее
releaseTags.forEach { releaseTag ->
    def releaseName = GitTags.sanitizeTagFromPrefixes(releaseTag, GitTags.getReleasePrefixes())
    def releaseCommitSha = System.findGlobalProperty("ci.gitea.release.commit.short-sha")
        .orElse(Git.commitShortSha)

    def releaseDescription = ReleaseDescriptionReader.readDescription(releaseName)
        .orElse("")

    giteaRepository.createOrUpdateRelease(releaseTag, releaseCommitSha, releaseName, releaseDescription, artifactLocalFiles)
}